/*****************************************************************************
 *
 * FILE:	api.c
 * DESCRIPTION:	API to convert kanji code
 * DATE:	Sun, Jan 16 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.com
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2000 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: api.c,v 1.2.2.4 2001/01/12 07:58:43 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */
#include <stdio.h>
#include "kanji.h"
#include "functions.h"


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static conv_t	convertKanjiCodeOfFile(kcode_t, FILE *, kcode_t, FILE *);
static kcode_t	ktype2kcode(ktype_t);
static void	printGuessedKanjiCode(ktype_t, bool_t);


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	wkfPrintGuessKanjiCodeOfFile
 * DESCRIPTION:		print the guessed kanji code of file
 * ARGUMENTS:		input file pointer, show detail flag
 * RETURN VALUE:	none
 */
void
wkfPrintGuessedKanjiCodeOfFile(fin, detail)
	FILE *	fin;
	bool_t	detail;
{
  readFile(fin);
  printGuessedKanjiCode(guessKanjiCodeType(TRUE, KC_UNKNOWN, NULL), detail);
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	wkfGuessKanjiCodeOfFile
 * DESCRIPTION:		guess kanji code of file
 * ARGUMENTS:		input file pointer
 * RETURN VALUE:	guessed kanji code
 */
kcode_t
wkfGuessKanjiCodeOfFile(fin)
	FILE *	fin;
{
  readFile(fin);
  return ktype2kcode(guessKanjiCodeType(TRUE, KC_UNKNOWN, NULL));
}

/*
 * FUNCTION NAME:	wkfGuessKanjiCodeOfString
 * DESCRIPTION:		guess kanji code of string
 * ARGUMENTS:		input string pointer
 * RETURN VALUE:	guessed kanji code
 */
kcode_t
wkfGuessKanjiCodeOfString(str)
	STR	str;
{
  readString(str);
  return ktype2kcode(guessKanjiCodeType(FALSE, KC_UNKNOWN, NULL));
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	wkfConvertKanjiCodeOfString
 * DESCRIPTION:		interface to convert kanji code of string
 * ARGUMENTS:		input kanji code, input string,
 *			output kanji code, output buffer,
 *			output buffer size
 * RETURN VALUE:	converted:    CONV_OK,
 *			no-converted: CONV_NO,
 *			otherwise:    CONV_ERR
 */
conv_t
wkfConvertKanjiCodeOfString(kin, sin, kout, sout, size)
	kcode_t	kin;
	STR	sin;
	kcode_t	kout;
	STR	sout;
	uint	size;
{
  readString(sin);
  /* guess the input kanji code automatically */
  if (kin == KC_UNKNOWN) {
    kin = ktype2kcode(guessKanjiCodeType(FALSE, kout, NULL));
  }

  setOutputStringBuffer(sout, size);

  return convertKanjiCode(kin, kout);
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	wkfFileOpen
 * DESCRIPTION:		open a file while converting kanji code
 * ARGUMENTS:		filename, kanji code
 * RETURN VALUE:	Success: file pointer, otherwise: NULL
 */
FILE *
wkfFileOpen(fname, kcode)
	STR	fname;
	kcode_t	kcode;
{
  register FILE *	fin;
  register FILE *	fout;

  /* open file */
  fin = fopen(fname, "r");
  if (fin == NULL) {
    return NULL;
  }

  /* open temporary file */
  fout = tmpfile();
  if (fout == NULL) {
    fclose(fin);
    return NULL;
  }

  /* convert file */
  switch (convertKanjiCodeOfFile(KC_UNKNOWN, fin, kcode, fout)) {
    case CONV_OK:
      fclose(fin);
      if (fseek(fout, 0L, SEEK_SET) == -1) {
	fclose(fout);
	return NULL;
      }
      return fout;
    case CONV_NO:
      fclose(fout);
      return fin;
    case CONV_ERR:
    default:
      fclose(fin);
      fclose(fout);
      return NULL;
  }
#if	0
  if (convertKanjiCodeOfFile(KC_UNKNOWN, fin, kcode, fout) == CONV_OK) {
    fclose(fin);
    if (fseek(fout, 0L, SEEK_SET) == -1) {
      fclose(fout);
      return NULL;
    }
    return fout;
  }
  else {	/* failsafe mode */
    fclose(fout);
    return fin;
  }
#endif
  return NULL;
}

/*
 * FUNCTION NAME:	wkfFileClose
 * DESCRIPTION:		close a stream which is opened by wkfFileOpen
 * ARGUMENTS:		file pointer
 * RETURN VALUE:	Success: 0, otherwise: EOF
 */
int
wkfFileClose(fp)
	FILE *	fp;
{
  return fclose(fp);
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	wkfConvertKanjiCodeOfFile
 * DESCRIPTION:		interface to convert kanji code
 * ARGUMENTS:		input kanji code, input file pointer,
 *			output kanji code, output file pointer
 * RETURN VALUE:	converted:    CONV_OK,
 *			no-converted: CONV_NO,
 *			otherwise:    CONV_ERR
 */
conv_t
wkfConvertKanjiCodeOfFile(kin, fin, kout, fout)
	kcode_t	kin;
	FILE *	fin;
	kcode_t	kout;
	FILE *	fout;
{
  return convertKanjiCodeOfFile(kin, fin, kout, fout);
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	convertKanjiCodeOfFile
 * DESCRIPTION:		interface to convert kanji code
 * ARGUMENTS:		input kanji code, input file pointer,
 *			output kanji code, output file pointer
 * RETURN VALUE:	converted:    CONV_OK,
 *			no-converted: CONV_NO,
 *			otherwise:    CONV_ERR
 */
static conv_t
convertKanjiCodeOfFile(kin, fin, kout, fout)
	kcode_t	kin;
	FILE *	fin;
	kcode_t	kout;
	FILE *	fout;
{
  /* guess the input kanji code automatically */
  if (kin == KC_UNKNOWN) {
    readFile(fin);
    kin = ktype2kcode(guessKanjiCodeType(TRUE, kout, fout));
  }
  else {
    noPreRead(fin);
  }

  setOutputFilePointer(fout);

  return convertKanjiCode(kin, kout);
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	ktype2kcode
 * DESCRIPTION:		convert ktype_t to kcode_t
 * ARGUMENTS:		guessed kanji code (ktype_t)
 * RETURN VALUE:	guessed kanji code (kcode_t)
 */
static kcode_t
ktype2kcode(kcode)
	ktype_t	kcode;
{
  switch (kcode) {
    case KT_EUC_JP_or_SJIS:
      return KC_EUCorSJIS;
    case KT_ASCII:
      return KC_ASCII;
    case KT_JIS_X0201_ROMA:
    case KT_JIS_X0201_KANA:
    case KT_JIS_X0212:
    case KT_JIS_OLD:
    case KT_JIS_OLD_X0201:
    case KT_JIS_OLD_X0212:
    case KT_JIS_NEW:
    case KT_JIS_NEW_X0201:
    case KT_JIS_NEW_X0212:
    case KT_JIS_1990:
    case KT_JIS_1990_X0212:
      return KC_JIS;
    case KT_EUC_JP:
    case KT_EUC_JP_X0212:
      return KC_EUC;
    case KT_SJIS:
      return KC_SJIS;
    case KT_JIS_OLD_BROKEN:
    case KT_JIS_NEW_BROKEN:
    case KT_JIS_1990_BROKEN:
    case KT_EUC_JP_BROKEN:
    case KT_SJIS_BROKEN:
      return KC_BROKEN;
    case KT_DATA:
      return KC_DATA;
    case KT_UNKNOWN:
    default:
      return KC_UNKNOWN;
  }
}


/*
 * FUNCTION NAME:	printGuessedKanjiCode
 * DESCRIPTION:		print the guessed kanji code
 * ARGUMENTS:		guessed kanji code, show detail flag
 * RETURN VALUE:	none
 */
static void
printGuessedKanjiCode(kcode, detail)
	ktype_t	kcode;
	bool_t	detail;
{
  switch (kcode) {
    case KT_EUC_JP_or_SJIS:
      fputs(detail ? "Japanese EUC or Shift-JIS\n" : "euc-jp/sjis\n", stdout);
      break;
    case KT_ASCII:
      fputs(detail ? "ASCII\n" : "ascii\n", stdout);
      break;
    case KT_JIS_X0201_ROMA:
      fputs(detail ? "JIS X 0201-1976 (roma)\n" : "jisx0201(roma)\n", stdout);
      break;
    case KT_JIS_X0201_KANA:
      fputs(detail ? "JIS X 0201-1976 (kana)\n" : "jisx0201(kana)\n", stdout);
      break;
    case KT_JIS_X0212:
      fputs(detail ? "JIS X 0212-1990\n" : "jisx0212\n", stdout);
      break;
    case KT_JIS_OLD:
      fputs(detail ? "JIS C 6226-1978\n" : "oldjis\n", stdout);
      break;
    case KT_JIS_OLD_X0201:
      fputs(detail ? "JIS C 6226-1978 with JIS X 0201 (kana)\n" :
		     "oldjis+jisx0201\n", stdout);
      break;
    case KT_JIS_OLD_X0212:
      fputs(detail ? "JIS C 6226-1978 with JIS X 0212-1990\n" :
		     "oldjis+jisx0212\n", stdout);
      break;
    case KT_JIS_OLD_BROKEN:
      fputs(detail ? "JIS C 6226-1978 with some BROKEN codes\n" :
		     "oldjis(broken)\n", stdout);
      break;
    case KT_JIS_NEW:
      fputs(detail ? "JIS X 0208-1983\n": "newjis\n", stdout);
      break;
    case KT_JIS_NEW_X0201:
      fputs(detail ? "JIS X 0208-1983 with JIS X 0201 (kana)\n" :
		     "newjis+jisx0201\n", stdout);
      break;
    case KT_JIS_NEW_X0212:
      fputs(detail ? "JIS X 0208-1983 with JIS X 0212-1990\n" :
		     "newjis+jisx0212\n", stdout);
      break;
    case KT_JIS_NEW_BROKEN:
      fputs(detail ? "JIS X 0208-1983 with some BROKEN codes\n" :
		     "newjis(broken)\n", stdout);
      break;
    case KT_JIS_1990:
      fputs(detail ? "JIS X 0208-1990\n" : "jis1990\n", stdout);
      break;
    case KT_JIS_1990_X0212:
      fputs(detail ? "JIS X 0208-1990 with JIS X 0212-1990\n" :
		     "jis1990+jisx0212\n", stdout);
      break;
    case KT_JIS_1990_BROKEN:
      fputs(detail ? "JIS X 0208-1990 with some BROKEN codes\n" :
		     "jis1990(broken)\n", stdout);
      break;
    case KT_EUC_JP:
      fputs(detail ? "Japanese EUC\n" : "euc-jp\n", stdout);
      break;
    case KT_EUC_JP_X0212:
      fputs(detail ? "Japanese EUC with JIS X 0212-1990\n" :
		     "euc-jp+jisx0212\n", stdout);
      break;
    case KT_EUC_JP_BROKEN:
      fputs(detail ? "Japanese EUC with some BROKEN codes\n" :
		     "euc-jp(broken)\n", stdout);
      break;
    case KT_SJIS:
      fputs(detail ? "Shift-JIS\n" : "sjis\n", stdout);
      break;
    case KT_SJIS_BROKEN:
      fputs(detail ? "Shift-JIS with some BROKEN codes\n" :
		     "sjis(broken)\n", stdout);
      break;
    case KT_DATA:
      fputs(detail ? "Binary Data\n" : "data\n", stdout);
      break;
    default:
      fputs(detail ? "Unknown\n" : "unknown\n", stdout);
  }
}

